/*
  ==============================================================================

    This file was auto-generated!

  ==============================================================================
*/

#pragma once

#include "../JuceLibraryCode/JuceHeader.h"

class SimpleTreeItem : public TreeViewItem
{
public:
    SimpleTreeItem()
    {
        setOpen(true);
    }

    bool mightContainSubItems() override
    {
        return getNumSubItems() != 0;
    }

    void paintItem(Graphics& g, int width, int height) override
    {
        g.fillAll(Colours::grey);
        g.setColour(Colours::black);
        g.drawText("item " + String(getRowNumberInTree()), 0, 0, width, height, Justification::left);
    }

    int getItemHeight() const
    {
        //Keep track of how many times we are calling TreeViewItem::getItemHeight.
        //Use a global variable for that.
        //This value is much smaller in JUCE 6.0.7 than in JUCE 6.1.2.
        getItemHeightCallCtr++;

        return TreeViewItem::getItemHeight();
    }

    static int getItemHeightCallCtr;


    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(SimpleTreeItem)
};


class TreeHolder : public Component
{
public:
    TreeHolder()
    {
        addAndMakeVisible(tree);
        tree.setRootItem(new SimpleTreeItem());
    }

    void paint(Graphics& g) override { g.fillAll(Colours::grey); }

    ~TreeHolder()
    {
        tree.deleteRootItem();
    }

    void resized() override 
    { 
        tree.setBounds(getLocalBounds()); 
    }

    void addChildren(int numChildren)
    {
        for (int i = 0; i < numChildren; ++i)
            tree.getRootItem()->addSubItem(new SimpleTreeItem());
    }

    TreeView tree;
};


//==============================================================================
/*
    This component lives inside our window, and this is where you should put all
    your controls and content.
*/
class MainComponent   : public Component
{
public:
    //==============================================================================
    MainComponent()
    {
        setSize(600, 400);

        myButton.setButtonText("Create Tree");
        addAndMakeVisible(myButton);
        myButton.onClick = [this]()
        {
            SimpleTreeItem::getItemHeightCallCtr = 0;
            treeHolder = std::make_unique<TreeHolder>();
            addAndMakeVisible(*treeHolder);
            resized();
            treeHolder->addChildren(1000);
            counterLabel.setText("Calls to getItemHeight = " + String(SimpleTreeItem::getItemHeightCallCtr), dontSendNotification);
        };

        addAndMakeVisible(counterLabel);

    }
    ~MainComponent() {}

    //==============================================================================
    void paint (Graphics& g) override
    {
        g.fillAll(getLookAndFeel().findColour(ResizableWindow::backgroundColourId));
    }

    void resized() override
    {
        myButton.setBounds(50, 150, 200, 50);
        counterLabel.setBounds(50, 250, 200, 50);

        if (treeHolder != nullptr)
        {
            auto bounds = getLocalBounds();
            bounds = bounds.reduced(10);
            bounds.removeFromLeft(bounds.getWidth() / 2);
            treeHolder->setBounds(bounds);
        }
    }

private:

    TextButton myButton;
    std::unique_ptr<juce::FileChooser> dirChooser;
    Label counterLabel; 

    std::unique_ptr<TreeHolder> treeHolder = nullptr;

    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MainComponent)
};
